/*
    Copyright 2004 James Nash
                                                                                                
    This file is part of Ganesha.
                                                                                                
    Ganesha is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                                
    Ganesha is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
                                                                                                
    You should have received a copy of the GNU General Public License
    along with Ganesha (see the LICENSE file); if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
                                                                                                
*/
package com.twiddles.ganesha.functionality;

import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.vocabulary.*;
/**
   Adaptor class for the Jena implementations of the RDF Containers.
   Provides common methods for the RDF Container implementations.

   @see JenaBag
   @see JenaSeq
   @see JenaAlt
   @author James Nash
*/
public abstract class JenaContainerAdaptor implements RDFContainer{

    /**
       The underlying Jena container.
    */
    protected Container con;


    /**
       Can be used by subclasses
       to construct an RDFContainer that holds a
       Jena Container object.

       @param con the Jena Container to use
    */
    public JenaContainerAdaptor(Container con){
	this.con = con;
    }


    /**
       Returns the number of objects in the container.
       
       @return number of objects
    */
    public int getSize(){
	return con.size();
    }


    /**
       Retrieves the object at the specified index in this container.
       
       @param index the index of the element to be retrieved
       @return objects in this container
    */
    public RDFObject getObject(int index){

	RDFObject output = null;

	RDFNode node = con.getProperty( RDF.li(index) ).getObject();
	
	if ( node instanceof Literal ){
	    Literal lit = (Literal) node;
	    output = new JenaObject( lit.getString() );
	}
	else{
	    output = new JenaObject( node );
	}

	return output;
    }


    /**
       Inserts a new object into this container.
       If this is a SEQ the object will be added at
       the end, if it is a ALT the new object will
       <em>not</em> be made default.

       @param object the object to insert
    */
    public void addObject(RDFObject object){

	// this is the Jena implementation so...
	JenaObject myObj = (JenaObject) object;

	con.add( myObj.getJenaNode() );
    }

  
  
  
    /**
       Removes the object at the specified position from the container.
       This method needs to be implemented by subclasses.

       @param index the index of the element to be deleted.
    */
    public abstract void removeObject(int index);


    /**
       Deletes this container.
    */
    public void delete(){
	// a jena container is also a jena resource so...
	con.removeProperties();
    }

    /**
        Swaps the object at the given index with the given new object.
        
        @param index the index of the old object
        @param object the new object to be placed at that index
    */
    public void setObjectAt(int index, RDFObject object){
    
        JenaObject jobj = (JenaObject) object;
        con.getProperty( RDF.li(index) ).changeObject( jobj.getJenaNode() );
    
    }


    /**
        Returns the URI of this container.
	Since containers are really just resources with
	a special rdf:type property they may have a URI.
	Unless they are anonymous, then they are assigned
	a unique ID.

	@see #isAnonymous()
	@return this resource's URI
    */
    public String getURI(){
	String out;
        if ( con.isAnon() ){
	    out = con.getId().toString();
        }
        else{
	    out = con.getURI();
        }
    
        return out;
    }


    /**
       Checks if this is an anonymous container.

       @return if anonymous or not
    */
    public boolean isAnonymous(){

	return con.isAnon();

    }


    /**
       Retrieves the underlying Jena Container object

       @return Jena Container
    */
    public Container getJenaNode(){
	return con;
    }

}
