/*
    Copyright 2004 James Nash
                                                                                                
    This file is part of Ganesha.
                                                                                                
    Ganesha is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                                
    Ganesha is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
                                                                                                
    You should have received a copy of the GNU General Public License
    along with Ganesha (see the LICENSE file); if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
                                                                                                
*/
package com.twiddles.ganesha.functionality;

import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.vocabulary.*;
import java.util.*;
/**
   Implementation of {@link RDFResource} using Jena.
   <p>
   This requires the <a href="http://jena.sourceforge.net/">Jena API</a>
   to be installed on your system and listed in java's CLASSPATH.
   
   @see JenaProperty
   @see JenaPredicate
   @see JenaObject
   @author James Nash
*/
public class JenaResource implements RDFResource{

    // The statements of this resource.
    private Resource res;
    
    // this resource's type property (if any)
    private Statement prop;


    /**
       Constructs a new JenaResource object.
       It effectively wraps a Resource object of the Jena API with
       the {@link RDFResource} interface so that Ganesha's
       GUI can interact with it.

       @param res the RDF resource's data
    */
    public JenaResource(Resource res){
        this.res = res;

	// determine type
	prop = null;
	if ( res.hasProperty( RDF.type ) ){
	    prop = res.getProperty( RDF.type );
	}

    }

    /**
        Returns the URI of this resource.
	If the resource is anonymous this will return
	some internal identifier to differentiate this
	anonymous resource from others.

	@see #isAnonymous()
	@return this resource's URI
    */
    public String getURI(){
        String out;
        if ( res.isAnon() ){
	    out = res.getId().toString();
        }
        else{
	    out = res.getURI();
        }
    
        return out;
    
    }

    


    /**       
       Return all the properties describing this resource.

       @return array of the properties
    */
    public RDFProperty[] getProperties(){

	StmtIterator stIt = res.listProperties();

	Vector tmp = new Vector();
	Statement tmpS;
	while ( stIt.hasNext() ){
	    tmpS = stIt.nextStatement();
	    
	    // don't want to add type statement
	    if ( tmpS.getPredicate().getURI() == RDF.type.getURI() ){
		// System.out.println("Ignoring type property");
	    }
	    else{
		tmp.add( tmpS );
	    }
	}

	RDFProperty[] stmts = new JenaProperty[ tmp.size() ];

	for (int i=0; i < stmts.length; i++){
	    stmts[i] = new JenaProperty( (Statement) tmp.elementAt(i) );
	}

	return stmts;
    }



    /**
       Adds a property describing this resource.

       @param prop the predicate (aka property) of the statement to be added
       @param obj the object (aka value) of the statement to be added
    */
    public void addProperty(RDFPredicate prop, RDFObject obj){
	
	// the inputs to the parameters must be Jena implementations
	JenaPredicate jprop = (JenaPredicate) prop;
	JenaObject jobj = (JenaObject) obj;

	res.addProperty( jprop.getJenaProp() , jobj.getJenaNode() );
    }

    /**
        Deletes all properties of this resource.
    */
    public void delete(){
	// if all the resource's properties are
	// removed Jena considers the resource removed too.
	Resource ignored = res.removeProperties();
    }

    /**
       Checks if this is an anonymous resource.

       @return if anonymous or not
    */
    public boolean isAnonymous(){
	return res.isAnon();
    }


    /**
       Returns a reference to the Jena API Resource inside this JenaResource object.
       This is for use by the other Jena* classes in this package.

       @return reference to Resource object
    */
    public Resource getJenaRes(){
	return res;
    }


    /**
       Returns the type of this resource.
       If the resource has an rdf:type property
       whose object is a resource then that resource's
       URI is returned. In any other case null will
       be returned.

       @return resource's type
    */
    public String getType(){
	String output = null;

	if ( prop != null ){
	    output = prop.getResource().getURI();
	}
	
	return output;
    }


    /**
       Returns a short description of the type of this resource.
       If this resource has no type null will be returned

       @return description of resource's type
    */
    public String getTypeDescr(){
	String output = null;

	if ( prop != null ){

	    output = JenaFile.shorten( prop.getResource().getURI() );
	}

	return output;
    }



    /**
       Sets the type of this resource.
       An rdf:type property with the resource
       specified by the given URI will be
       added to this reosurce.

       @param uri the URI identifying this type.
    */
    public void setType(String uri){
	Model tmp = ModelFactory.createDefaultModel();
	Resource tmpR = tmp.createResource( uri );
	// if a property is present change it
	if ( prop != null ){
	    // delete old one
	    prop.remove();
	    
	    // add new one
	    res.addProperty( RDF.type, tmpR );

	    prop = res.getProperty( RDF.type );
	}
	else{
	    // if not add it
	    res.addProperty( RDF.type, tmpR );

	    prop = res.getProperty( RDF.type );
	}
    }

    /**
       Removes the type property of this resource.
    */
    public void delType(){
	if ( prop != null ){
	    // there has to be a property before it can be deleted!
	    prop.remove();

	}
	prop = null;
    }

}
