/*
    Copyright 2004 James Nash
                                                                                                
    This file is part of Ganesha.
                                                                                                
    Ganesha is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                                
    Ganesha is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
                                                                                                
    You should have received a copy of the GNU General Public License
    along with Ganesha (see the LICENSE file); if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
                                                                                                
*/
package com.twiddles.ganesha.functionality;

/**
   Interface for querying and modifying RDF containers.
   A container is really just a special kind of anonymous
   resource with an additional implied meaning. There are
   3 types of containers: <em>bags</em>, <em>sequences</em>
   and <em>alternatives</em>.
   <p>
   A bag represents an unordered set of properties. A sequence
   represents properties that have a particular order. A set
   of alternatives is essentially a bag, except that one property is
   consdered the default.
   <p>
   This interface should not be implemented directly, instead use
   the appropriate sub-interface.

   @see RDFBag
   @see RDFSeq
   @see RDFAlt
   @author James Nash
*/
public interface RDFContainer{

    /**
       Returns the number of objects in the container.
       
       @return number of objects
    */
    int getSize();


    /**
       Retrieves the object at the specified index in this container.
       
       @param index the index of the element to be retrieved
       @return objects in this container
    */
    RDFObject getObject(int index);


    /**
       Inserts a new object into this container.
       If this is a SEQ the object will be added at
       the end, if it is a ALT the new object will
       <em>not</em> be made default.

       @param object the object to insert
    */
    void addObject(RDFObject object);

    /**
       Removes the object at the specified position from the container.

       @param index the index of the element to be deleted.
    */
    void removeObject(int index);

    /**
        Swaps the object at the given index with the given new object.
        
        @param index the index of the old object
        @param object the new object to be placed at that index
    */
    void setObjectAt(int index, RDFObject object);



    /**
       Deletes this container.
    */
    void delete();


    /**
        Returns the URI of this container.
	Since containers are really just resources with
	a special rdf:type property they may have a URI.
	Unless they are anonymous, then they are assigned
	a unique ID.

	@see #isAnonymous()
	@return this resource's URI
    */
    String getURI();


    /**
       Checks if this is an anonymous container.

       @return if anonymous or not
    */
    boolean isAnonymous();


}
