/*
    Copyright 2004 James Nash
                                                                                                
    This file is part of Ganesha.
                                                                                                
    Ganesha is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                                
    Ganesha is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
                                                                                                
    You should have received a copy of the GNU General Public License
    along with Ganesha (see the LICENSE file); if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
                                                                                                
*/
package com.twiddles.ganesha.functionality;

/**
   Interface for querying the object of an RDF resource's property.
   The object of a property statement about a resource can be
   a literal value, another resource or a container.

   @see RDFResource
   @see RDFContainer
   @author James Nash
*/
public interface RDFObject{


    /**
       Indicates that this object is a literal.
    */
    public static final int LITERAL = 0;

    /**
       Indicates that this object is a resource.
    */
    public static final int RESOURCE = 1;

    /**
       Indicates that this object is a bag.
    */
    public static final int BAG = 2;

    /**
       Indicates that this object is a seq.
    */
    public static final int SEQ = 3;

    /**
       Indicates that this object is a alt.
    */
    public static final int ALT = 4;

    /**
       Indicates that this object is of an unsupported type.
    */
    public static final int UNKNOWN = 5;

    /**
       Query the type of this RDF object.
       The returned value can be compared to the
       constants defined in this interface to determine
       of what type the object is.

       @see #LITERAL
       @see #RESOURCE
       @see #BAG
       @see #SEQ
       @see #ALT
       @see #UNKNOWN
       @return object type code
    */
    int getObjType();


    /**
       Retrieve the literal value.
       If this object is of type literal then the value
       will be returned. Otherwise null will be returned.

       @return value or null
    */
    String getLiteral();


    /**
       Retrieve resource.
       If this object is of type resource then the
       resource will be returned. Otherwise null will be returned.

       @return resource or null
    */
    RDFResource getResource();


    /**
       Retrieve bag.
       If this object is of type bag then
       that container will be returned. Otherwise null
       will be returned.

       @return bag or null
    */
    RDFBag getBag();

    /**
       Retrieve sequence.
       If this object is of type seq then
       that container will be returned. Otherwise null
       will be returned.

       @return sequence or null
    */
    RDFSeq getSeq();

    /**
       Retrieve alternatives.
       If this object is of type alt then
       that container will be returned. Otherwise null
       will be returned.

       @return alternatives or null
    */
    RDFAlt getAlt();


    /**
       Return a unique ID for this object.
       Use this to compare to objects.

       @return unique id string for this object
    */
    String getID();

}
