/*
    Copyright 2004 James Nash
                                                                                                
    This file is part of Ganesha.
                                                                                                
    Ganesha is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                                
    Ganesha is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
                                                                                                
    You should have received a copy of the GNU General Public License
    along with Ganesha (see the LICENSE file); if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
                                                                                                
*/
package com.twiddles.ganesha.functionality;

/**
   Interface for querying and modifying RDF resources.
   A resource may have some properties describing it.
   Each property statement in RDF has 3 parts: the <em>subject</em>
   which is the resource being described, the <em>predicate</em>
   which is the type of property of the resource being described and
   the <em>object</em> which is the value of that property.
   <p>
   A resource is usually identified by a URI, unless it is an
   anonymous resource. An anonymous resource implies that it
   is a thing that has properties but does not (yet) have a URI
   that identifies it. Anonymous resources are usually the
   object of other resource's properties.

   @see RDFProperty
   @see RDFPredicate
   @see RDFObject

   @author James Nash
 */
public interface RDFResource {

    /**
        Returns the URI of this resource.
	If the resource is anonymous this will return
	some internal identifier to differentiate this
	anonymous resource from others.

	@see #isAnonymous()
	@return this resource's URI
    */
    String getURI();

    /**
       Returns all the properties describing this resource.

       @return array of the properties
    */
    RDFProperty[] getProperties();

    /**
       Adds a property describing this resource.

       @param prop the predicate (aka property) of the statement to be added
       @param obj the object (aka value) of the statement to be added
    */
    void addProperty(RDFPredicate prop, RDFObject obj);


    /**
       Deletes all properties of this resource.
    */
    void delete();

    /**
       Checks if this is an anonymous resource.

       @return if anonymous or not
    */
    boolean isAnonymous();


    /**
       Returns the type of this resource.
       If the resource has an rdf:type property
       whose object is a resource then that resource's
       URI is returned. In any other case null will
       be returned.

       @return resource's type
    */
    String getType();

    /**
       Returns a short description of the type of this resource.
       If this resource has no type null will be returned

       @return description of resource's type
    */
    String getTypeDescr();


    /**
       Sets the type of this resource.
       An rdf:type property with the resource
       specified by the given URI will be
       added to this reosurce.

       @param uri the URI identifying this type.
    */
    void setType(String uri);

    /**
       Removes the type property of this resource.
    */
    void delType();



}
