/*
    Copyright 2004 James Nash
                                                                                                
    This file is part of Ganesha.
                                                                                                
    Ganesha is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                                
    Ganesha is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
                                                                                                
    You should have received a copy of the GNU General Public License
    along with Ganesha (see the LICENSE file); if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
                                                                                                
*/
package com.twiddles.ganesha.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import com.twiddles.ganesha.functionality.*;
import com.twiddles.ganesha.gui.gfx.*;
/**
	Wizard interface to let users add new resources, properties or objects.
	The methods in this class will display a wizard prompting users for
	information needed to create a new resource, property or object on a
	step by step basis. Users have the option to go back and ammed or change
	any values entered in previous steps or to cancel the the process and return
	to the main Ganesha GUI.

	@author James Nash
 */
class AddWizard{

    // window dimensions
    private final int WIDTH = 400;
    private final int HEIGHT = 270;

    // rdf data
    private RDFFile rdfFile;
    private GUI gui;

    // the stuff that gets created
    private RDFResource res;
    private RDFPredicate pred;
    private RDFObject obj;


    // next button
    private JButton nextButton;

    // previous button
    private JButton prevButton;


    // dialog window containing the wizard
    private JDialog window;

    // the dialog's content pane
    private Container pane;


    // stuff for subject screen...
    private int choice;
    private JTextField entry;
    private final String initTxt = "http://";
    

    // stuff from predicate screen...
    private int predType;


    // stuff from 1st object screen...
    private int objType;

    // stuff from 2nd object screens....
    private int conType;
    private JTextArea textbox;
    private JTextField resEntry;
    private int resIndex;
    private String resURI;
    private int resType;
    private JComboBox resChooser;

    // which screen did we start with?
    private int start;
    // which screen are we on?
    private int screen;

    /**
       Constructs a new wizard instance.
       
       @param gui a reference to the main app window
     */
    public AddWizard( GUI gui ){

	res = null;
	pred = null;
	obj = null;

	// set some initial options
	objType = 1;
	conType = 1;
	resType = 1;
	resIndex = 0;
	resURI = initTxt;

	// get reference to rdf data
	this.gui = gui;
	this.rdfFile = gui.getRDF();

	// create modal dialogue window
	window = new JDialog( gui, true );

	pane = window.getContentPane();

	// set up buttons
	nextButton = new JButton( "Next", GUIIcons.getIcon( GUIIcons.MINI_FORWARD_ICON ) );
	prevButton = new JButton( "Previous", GUIIcons.getIcon( GUIIcons.MINI_BACK_ICON ));

	JButton cancelButton = new JButton( "Cancel" );

	Box buttons = Box.createHorizontalBox();

	buttons.add( Box.createHorizontalGlue() );
	buttons.add( prevButton );
	buttons.add( nextButton );
	buttons.add( Box.createHorizontalStrut( 5 ) );
	buttons.add( cancelButton );
	


	// make cancel button close window!
	cancelButton.addActionListener( new ActionListener() {

		public void actionPerformed(ActionEvent e){
		    // clear any values
		    res = null;
		    pred = null;
		    obj = null;

		    // close the dialog
		    window.dispose();
		}

	    });

	// do the same if the user closes the dialog window
	window.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
	// we'll handle closing ourselves, thank you very much!
	window.addWindowListener( new WindowAdapter() {

	        public void windowClosing(WindowEvent w){
		    // clear any values
		    res = null;
		    pred = null;
		    obj = null;

		    // tell Ganesha to shutdown
		    window.dispose();
		    
		}

	    });



	// monster actionlistener for next button!
	nextButton.addActionListener( new ActionListener(){

		public void actionPerformed(ActionEvent e){
		    
		    // which screen are we on?
		    switch( screen ){
			
		    case 1: // subject screen
			// what kind of resource to make?
			if ( choice == 1 ){
			    // described resource!
			    
			    // did the user enter a URI?
			    String uri = entry.getText().trim();
			    if (( uri.length() != 0 ) && ( ! uri.equals( initTxt ) )){
				// yes!
				res = rdfFile.createResource( uri );
				// proceed to next screen
				swapPanel( createAddPredicatePanel() );
				screen++;
			    }
			    else{
				// alert user...
				JOptionPane.showMessageDialog(window, "You must type in a valid URI for the resource\nyou wish to describe in order to proceed.\n\nIf you cannot do so use the 'Anonymous' option.", "No URI provided", JOptionPane.WARNING_MESSAGE );
			    }
			    
			}
			else{
			    // anonymous!
			    // create it...
			    res = rdfFile.createResource();
			    
			    // proceed to next screen
			    swapPanel( createAddPredicatePanel() );
			    screen++;
			}
			break;

			
		    case 2: // predicate screen
			// a predicate should be set in any case so no further
			// checking is necessary...
			swapPanel( createAddObjectPanel() );
			screen++;
			break;

		    case 3: // object screen #1
			// what kind of object did the user pick?
			switch( objType ){
			    
			case 1: // literal
			    swapPanel( createAddLiteralObjectPanel() );
			    screen++;
			    break;

			case 2: // resource
			    swapPanel( createAddResourceObjectPanel() );
			    screen++;
			    break;

			case 3: // container
			    swapPanel( createAddContainerObjectPanel() );
			    screen++;
			    break;

			}
			// on last screen the next button should become finish
			nextButton.setText("Finish");
			nextButton.setIcon( GUIIcons.getIcon( GUIIcons.MINI_FINISH_ICON ) );
			break;

		    case 4: // object screen #2 (this is when the user clicks 'finish'
			// first figure out what kind of object we're making
			switch( objType ){

			case 1: // literal
			    //System.out.println("Make Literal!");

			    // get cleaned-up user input
			    String literalValue = textbox.getText().trim().replaceAll("[ \t]+"," ");

			    if ( literalValue.equals("") ){
				// nothing was entered
				// alert user...
				JOptionPane.showMessageDialog(window, "You can't add an empty value.\nPlease type something into the box.", "Empty literal value", JOptionPane.WARNING_MESSAGE );

			    }
			    else{
				obj = rdfFile.createObject( literalValue );
				window.dispose();
			    }
			    break;

			case 2: // resource
			    //System.out.println("Make Resource!");
			    if ( resType == 1 ){
				// user supplied URI for new resource
				//System.out.println("Make custom Resource!");
				String uri = resEntry.getText().trim();
				if (( uri.length() != 0 ) && ( ! uri.equals( initTxt ) )){
				    // yes!
				    RDFResource myRes = rdfFile.createResource( uri );
				    
				    obj = rdfFile.createObject( myRes );
				    window.dispose();
				}
				else{
				    // alert user...
				    JOptionPane.showMessageDialog(window, "You must type in a valid URI for the resource\nin order to proceed.", "No URI provided", JOptionPane.WARNING_MESSAGE );
				}


			    }
			    else{
				// user picked existing resource
				RDFResource myRes = (RDFResource) resChooser.getSelectedItem();
				obj = rdfFile.createObject( myRes );
				window.dispose();
			    }


			    break;

			case 3: // container
			    //System.out.println("Make Container!");
			    // what sort of container?
			    switch ( conType ){

			    case 1: // bag
				obj = rdfFile.createObject( rdfFile.createBag() );
				window.dispose();
				break;

			    case 2: // seq
				obj = rdfFile.createObject( rdfFile.createSeq() );
				window.dispose();
				break;

			    case 3: // alt
				obj = rdfFile.createObject( rdfFile.createAlt() );			
				window.dispose();
				break;

			    }
			    break;
			}
			// and we're done!
			break;

		    default: // should never happen!
			System.out.println("What the fuck just happened???");
			break;

		    }
		    
		}

	    });

	// make previous button work!
	prevButton.addActionListener( new ActionListener() {

		public void actionPerformed(ActionEvent e){
		    
		    // which screen are we on?
		    switch( screen ){
			
		    case 2: // predicate screen
			swapPanel( createAddSubjectPanel() );
			screen--;
			break;
			
		    case 3: // object screen #1
			swapPanel( createAddPredicatePanel() );
			screen--;
			break;
			
		    case 4: // object screen #2
			swapPanel( createAddObjectPanel() );
			screen--;
			// if we are going back from the resource object screen
			// remember the user's entry...
			if ( objType == 2 ){
			    resURI = resEntry.getText();
			}
			// make the next button display 'next' again
			nextButton.setText("Next");
			nextButton.setIcon( GUIIcons.getIcon( GUIIcons.MINI_FORWARD_ICON ) );
			break;

		    default: // should never happen!
			System.out.println("What the fuck just happened???");
			break;

		    }
		}
	    });


	// the main surface
	// (just blank for now)
	JPanel tmp = new JPanel();


	// add buttons to dialog window
	pane.add( tmp, BorderLayout.CENTER );
        pane.add( buttons, BorderLayout.SOUTH );

	// set the size
	window.setSize(WIDTH, HEIGHT);

	// center it above the parent window...
	int xOffset = (( gui.getWidth() - WIDTH) / 2);	    
	int yOffset = (( gui.getHeight() - HEIGHT) / 2);
	window.setLocation( new Point(( gui.getX() + xOffset ), ( gui.getY() + yOffset )) );


    }




    /**
       Create the panel for choosing a resource subject.

       @return the subject panel
     */
    private JPanel createAddSubjectPanel(){

	//=================== make this panel ==============

	GridBagLayout layout = new GridBagLayout();

	JPanel panel = new JPanel(layout);
	
	panel.setBorder( BorderFactory.createCompoundBorder(
							   BorderFactory.createEmptyBorder(2,5,2,5),
							   BorderFactory.createCompoundBorder(
											      BorderFactory.createTitledBorder("Specify resource to describe"),
											      BorderFactory.createEmptyBorder(5,5,5,5)
											      )
							   )
			);

	GridBagConstraints gbc = new GridBagConstraints();
	gbc.gridwidth = 3;
	gbc.gridheight = 1;
	gbc.gridx = 0;
	gbc.gridy = 0;
	gbc.anchor = GridBagConstraints.NORTHWEST;
	gbc.fill = GridBagConstraints.NONE;

	JLabel message = new JLabel("If the resource can be uniquely identified");
	panel.add( message, gbc );

	gbc.gridy++;

	message = new JLabel("enter its URI, otherwise choose 'Anonymous'");
	panel.add( message, gbc );
	
	gbc.gridy++;

	// blank space
	message = new JLabel(" ");
	panel.add( message, gbc );

	// user choices
	ButtonGroup choices = new ButtonGroup();
	
	
	JRadioButton option1 = new JRadioButton("URI: ", true);
	choices.add( option1 );

	gbc.gridwidth = 1;
	gbc.gridx = 0;
	gbc.gridy++;
	gbc.fill = GridBagConstraints.NONE;
	gbc.anchor = GridBagConstraints.WEST;

	panel.add( option1, gbc );

	// text field for URI
	entry = new JTextField( initTxt, 18);

	gbc.gridwidth = 2;
	gbc.gridx = 1;

	panel.add( entry, gbc );

	JRadioButton option2 = new JRadioButton("Anonymous", false);
	choices.add( option2 );

	gbc.gridx = 0;
	gbc.gridy++;
	gbc.gridwidth = 3;

	panel.add( option2, gbc );

	// keep track of what kind of resource the user wants
	//
	// 1: described resource
	// 2: anonymous resource
	//
	// initally the URI box is selected
	choice = 1;

	// add some actions
	// if option1 is seleced then enable the uri box
	option1.addActionListener( new ActionListener(){

		public void actionPerformed( ActionEvent e ){
		    entry.setEnabled( true );
		    choice = 1;
		}

	    });

	// if option2 is seleced then disable the uri box
	option2.addActionListener( new ActionListener(){

		public void actionPerformed( ActionEvent e ){
		    entry.setEnabled( false );
		    choice = 2;
		}

	    });


	//=========== Set up next / prev buttons ==============

	// this panel is always the first so
	// disable previous button...
	prevButton.setEnabled( false );



	//============== init values / choices ==================
	
	if ( res != null ){
	    // this is not the first time this panel has been shown!
	    if ( res.isAnonymous() ){
		// 'click' the anonymous button
		option2.doClick();
	    }
	    else{
		// resource with URI
		entry.setText( res.getURI() );

	    }

	} 


	return panel;
    }






    /**
       Create the panel for choosing a predicate.

       @return the predicate panel
     */
    private JPanel createAddPredicatePanel(){

	//=================== make this panel ==============

	GridBagLayout layout = new GridBagLayout();
	JPanel panel = new JPanel(layout);

	panel.setBorder( BorderFactory.createCompoundBorder(
							   BorderFactory.createEmptyBorder(2,5,2,5),
							   BorderFactory.createCompoundBorder(
											      BorderFactory.createTitledBorder("Choose a property type"),
											      BorderFactory.createEmptyBorder(5,5,5,5)
											      )
							   )
			);


	GridBagConstraints gbc = new GridBagConstraints();
	gbc.gridwidth = 4;
	gbc.gridheight = 1;
	gbc.gridx = 0;
	gbc.gridy = 0;
	gbc.anchor = GridBagConstraints.NORTHWEST;
	gbc.fill = GridBagConstraints.BOTH;


	panel.add( new JLabel("Please choose a property of the"), gbc );
	gbc.gridy++;

	panel.add( new JLabel("resource that you wish to describe."), gbc );
	gbc.gridy++;

	// only add this if we are adding a whole new resource...
	if ( start < 2 ){
	    panel.add( new JLabel(" ") , gbc );
	    gbc.gridy++;
	    panel.add( new JLabel("(You can add further properties later on)"), gbc );
	    gbc.gridy++;
	}

	// blank space
	panel.add( new JLabel(" ") , gbc );
	gbc.gridy++;


	// the combo box
	gbc.gridwidth = 3;
	gbc.anchor = GridBagConstraints.WEST;

	final JComboBox chooser = new JComboBox( rdfFile.getPropTypes() );
	chooser.setRenderer( new PropTypeRenderer() );
	
	// add a listener to change pred when user chooses stuff from the list
	chooser.addActionListener( new ActionListener(){
		
		public void actionPerformed( ActionEvent e ){
		    
		    //JComboBox cBox = (JComboBox)e.getSource();
		    pred = (RDFPredicate) chooser.getSelectedItem();
		    predType = chooser.getSelectedIndex();

		}
	    });



	panel.add( chooser, gbc );

	// the add custom button
	gbc.gridwidth = 1;
	gbc.gridx = 3;

	JButton addButton = new JButton( "Custom type..." );
	addButton.setToolTipText("Add a custom property type...");


	// make the button prompt the user for a property URI
	addButton.addActionListener( new ActionListener(){

		public void actionPerformed( ActionEvent e ){

		    // ask user for type's URI
		    String output;

		    
		    JTextField textbox = new JTextField( "http://", 30);            
		    textbox.setEditable(true);
		    
		    JLabel label = new JLabel("Enter the URI of the property type:");
		    
		    Object[] contents = { label, textbox };
		    
		    int choice = JOptionPane.showConfirmDialog(window, contents, "Enter custom property type URI", JOptionPane.OK_CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE);
		    
		    if ( choice != JOptionPane.CANCEL_OPTION ){
			// user clicked OK
			// get the user input
			output = textbox.getText().trim();
			
			// add new poperty type to list
			chooser.addItem( rdfFile.createPredicate( output ) );
			// select it
			chooser.setSelectedIndex( ( chooser.getItemCount() - 1 ) );

			// set our pred to it
			pred = (RDFPredicate) chooser.getSelectedItem();
		    }
		    // if the user cancelled do nothing!
		}

	    });





	panel.add( addButton, gbc );



	//=========== Set up next / prev buttons ==============

	// if this is the first panel disable 'previous'
	prevButton.setEnabled( (start < 2) );



	//=========== init selection if this screen has appeared before ======
	if ( pred != null ){

	    // if the last choice is within range it was a standard choice
	    if ( predType < chooser.getItemCount() ){
		chooser.setSelectedIndex( predType );
	    }
	    else{
		// add the poperty type to list
		chooser.addItem( pred );
		// select it
		chooser.setSelectedIndex( ( chooser.getItemCount() - 1 ) );
	    }

	}
	// first time this screen appeared!
	else{
	    // initally pick dc:creator
	    chooser.setSelectedIndex( 17);
	    
	    // get that value
	    pred = (RDFPredicate) chooser.getSelectedItem();
	}

	return panel;
    }


    /**
       Create the panel for choosing an object.

       @return the object panel
     */
    private JPanel createAddObjectPanel(){

	//=================== make this panel ==============

	GridBagLayout layout = new GridBagLayout();
	JPanel panel = new JPanel(layout);

	panel.setBorder( BorderFactory.createCompoundBorder(
							   BorderFactory.createEmptyBorder(2,5,2,5),
							   BorderFactory.createCompoundBorder(
											      BorderFactory.createTitledBorder("Choose a value type"),
											      BorderFactory.createEmptyBorder(5,5,5,5)
											      )
							   )
			);


	GridBagConstraints gbc = new GridBagConstraints();
	gbc.gridwidth = 1;
	gbc.gridheight = 1;
	gbc.gridx = 0;
	gbc.gridy = 0;
	gbc.anchor = GridBagConstraints.NORTHWEST;
	gbc.fill = GridBagConstraints.NONE;


	// only add this if we previously chose a property type...
	if ( start < 3 ){
	    panel.add( new JLabel("Please select what kind of value you") , gbc );
	    gbc.gridy++;
	    panel.add( new JLabel("want to give this property."), gbc );
	    gbc.gridy++;
	}
	else{
	    // we must be adding objects to a container...
	    panel.add( new JLabel("Please choose what kind of value you") , gbc );
	    gbc.gridy++;
	    panel.add( new JLabel("want to add to the container."), gbc );
	    gbc.gridy++;

	}
	    

	// blank space
	panel.add( new JLabel(" ") , gbc );
	gbc.gridy++;


	// the combo box
	gbc.anchor = GridBagConstraints.WEST;

	ButtonGroup choices = new ButtonGroup();


	JRadioButton option1 = new JRadioButton("Literal value", true);
	choices.add( option1 );
	// makee clicking this button change the type
	option1.addActionListener( new ActionListener(){

		public void actionPerformed( ActionEvent e ){
		    objType = 1;
		}

	    });

	panel.add( option1 , gbc );
	gbc.gridy++;


	JRadioButton option2 = new JRadioButton("Another resource", false);
	choices.add( option2 );
	// makee clicking this button change the type
	option2.addActionListener( new ActionListener(){

		public void actionPerformed( ActionEvent e ){
		    objType = 2;
		}

	    });

	panel.add( option2 , gbc );
	gbc.gridy++;


	JRadioButton option3 = new JRadioButton("A container to hold several values", false);
	choices.add( option3 );
	// make clicking this button change the type
	option3.addActionListener( new ActionListener(){

		public void actionPerformed( ActionEvent e ){
		    objType = 3;
		}

	    });

	panel.add( option3 , gbc );
	gbc.gridy++;



	//=========== Set up next / prev buttons ==============

	// if this is the first screen disable previous, else enable it
	prevButton.setEnabled( (start < 3) );

	//========= set to last user choice if necessary... ==============
	
	if ( objType != 1 ){
	    
	    switch (objType){

	    case 2:
		option2.doClick();
		break;
		
	    case 3:
		option3.doClick();
		break;
	    }
	}


	return panel;
    }





    /**
       Create the panel for choosing an object's literal value.

       @return the literal object panel
     */
    private JPanel createAddLiteralObjectPanel(){

	//=================== make this panel ==============

	GridBagLayout layout = new GridBagLayout();
	JPanel panel = new JPanel(layout);

	panel.setBorder( BorderFactory.createCompoundBorder(
							   BorderFactory.createEmptyBorder(2,5,2,5),
							   BorderFactory.createCompoundBorder(
											      BorderFactory.createTitledBorder("Enter property value"),
											      BorderFactory.createEmptyBorder(5,5,5,5)
											      )
							   )
			);


	GridBagConstraints gbc = new GridBagConstraints();
	gbc.gridwidth = 1;
	gbc.gridheight = 1;
	gbc.gridx = 0;
	gbc.gridy = 0;
	gbc.anchor = GridBagConstraints.NORTHWEST;
	gbc.fill = GridBagConstraints.NONE;

	panel.add(new JLabel("Please type in a value for this property:"), gbc);
	gbc.gridy++;
	panel.add(new JLabel(" "), gbc);
	gbc.gridy++;

	// don't make a blank textbox if we already have an old one...
	if ( textbox == null ){
	    textbox = new JTextArea( "", 5, 30);
	    textbox.setLineWrap(true);
	    textbox.setWrapStyleWord(true);
	    textbox.setEditable(true);
	}

	JScrollPane scroller = new JScrollPane( textbox, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	panel.add( scroller, gbc);


	prevButton.setEnabled( true );
	return panel;
    }

    /**
       Create the panel for choosing an object's resource value.

       @return the resource object panel
     */
    private JPanel createAddResourceObjectPanel(){

	//=================== make this panel ==============

	GridBagLayout layout = new GridBagLayout();
	JPanel panel = new JPanel(layout);

	panel.setBorder( BorderFactory.createCompoundBorder(
							   BorderFactory.createEmptyBorder(2,5,2,5),
							   BorderFactory.createCompoundBorder(
											      BorderFactory.createTitledBorder("Pick a resource value"),
											      BorderFactory.createEmptyBorder(5,5,5,5)
											      )
							   )
			);


	GridBagConstraints gbc = new GridBagConstraints();
	gbc.gridwidth = 3;
	gbc.gridheight = 1;
	gbc.gridx = 0;
	gbc.gridy = 0;
	gbc.anchor = GridBagConstraints.NORTHWEST;
	gbc.fill = GridBagConstraints.NONE;

	// get currently described resources...
	RDFResource[] reses = rdfFile.getResources();

	if( reses.length > 0 ){
	    // there are some described resources...
	    panel.add( new JLabel("You can either specify a resource by its URI"), gbc );
	    gbc.gridy++;
	    
	    panel.add( new JLabel("or choose one of the other resources described"), gbc );	
	    gbc.gridy++;
	    
	    panel.add( new JLabel("in the current file."), gbc );	
	    gbc.gridy++;

	    
	    // blank space
	    panel.add( new JLabel(" "), gbc );

	    // user choices
	    ButtonGroup choices = new ButtonGroup();
	    
	    
	    JRadioButton option1 = new JRadioButton("URI: ", true);
	    choices.add( option1 );
	    option1.addActionListener( new ActionListener(){

		public void actionPerformed( ActionEvent e ){
		    resEntry.setEnabled( true );
		    resChooser.setEnabled( false );
		    resType = 1;
		}

	    });


	    gbc.gridwidth = 1;
	    gbc.gridx = 0;
	    gbc.gridy++;
	    gbc.fill = GridBagConstraints.NONE;
	    gbc.anchor = GridBagConstraints.WEST;
	    
	    panel.add( option1, gbc );
	    
	    // text field for URI
	    resEntry = new JTextField( resURI, 18);
	    
	    gbc.gridwidth = 2;
	    gbc.gridx = 1;
	    
	    panel.add( resEntry, gbc );
	    
	    JRadioButton option2 = new JRadioButton("", false);
	    choices.add( option2 );
	    option2.addActionListener( new ActionListener(){

		public void actionPerformed( ActionEvent e ){
		    resEntry.setEnabled( false );
		    resChooser.setEnabled( true );
		    resType = 2;
		}

	    });
	    
	    gbc.gridx = 0;
	    gbc.gridy++;
	    gbc.gridwidth = 1;

	    panel.add( option2, gbc );


	    gbc.gridx = 1;
	    gbc.gridwidth = 2;

	    resChooser = new JComboBox( reses );
	    resChooser.setRenderer( new ResourceRenderer() );
	    resChooser.setSelectedIndex( resIndex );
	    resChooser.addActionListener( new ActionListener(){

		    // update index if user picks something from the index
		public void actionPerformed( ActionEvent e ){
		    resIndex = resChooser.getSelectedIndex();
		}

	    });

	    // initally this option is unselected, hence disabled
	    resChooser.setEnabled( false );

	    panel.add( resChooser, gbc);


	    // init old value
	    if ( resType == 2 ){
		option2.doClick();
	    }

	}
	else{
	    panel.add( new JLabel("Please specify a resource by its URI."), gbc );
	    gbc.gridy++;
	    panel.add( new JLabel(" "), gbc );
	    gbc.gridy++;

	    gbc.gridwidth = 1;
	    panel.add( new JLabel("URI: ") , gbc);
	    
	    gbc.gridx = 1;
	    gbc.gridwidth = 2;
	    // text field for URI
	    resEntry = new JTextField( resURI, 18);

	    panel.add( resEntry, gbc );

	}





	prevButton.setEnabled( true );
	return panel;
    }




    /**
       Create the panel for choosing an object's container value.

       @return the container object panel
     */
    private JPanel createAddContainerObjectPanel(){

	//=================== make this panel ==============

	GridBagLayout layout = new GridBagLayout();
	JPanel panel = new JPanel(layout);

	panel.setBorder( BorderFactory.createCompoundBorder(
							   BorderFactory.createEmptyBorder(2,5,2,5),
							   BorderFactory.createCompoundBorder(
											      BorderFactory.createTitledBorder("Choose a container type"),
											      BorderFactory.createEmptyBorder(5,5,5,5)
											      )
							   )
			);


	GridBagConstraints gbc = new GridBagConstraints();
	gbc.gridwidth = 2;
	gbc.gridheight = 1;
	gbc.gridx = 0;
	gbc.gridy = 0;
	gbc.anchor = GridBagConstraints.NORTHWEST;
	gbc.fill = GridBagConstraints.NONE;

	panel.add( new JLabel("Please choose one of the following three") , gbc );
	gbc.gridy++;
	panel.add( new JLabel("kinds of container.") , gbc );
	gbc.gridy++;
	panel.add( new JLabel("( The container will initially be empty") , gbc );
	gbc.gridy++;
	panel.add( new JLabel("  but you can fill it with values later)") , gbc );
	gbc.gridy++;

	// blank space
	panel.add( new JLabel(" ") , gbc );
	gbc.gridy++;

	// the options...
	gbc.anchor = GridBagConstraints.WEST;
	ButtonGroup choices = new ButtonGroup();

	// 2 columns now - icon and radio button
	gbc.gridwidth = 1;

	panel.add( new JLabel( GUIIcons.getIcon( GUIIcons.MINI_BAG_ICON ) ), gbc );
	gbc.gridx++;

	JRadioButton option1 = new JRadioButton("Bag", true);
	choices.add( option1 );
	// make clicking this button change the type
	option1.addActionListener( new ActionListener(){

		public void actionPerformed( ActionEvent e ){
		    conType = 1;
		}

	    });

	panel.add( option1 , gbc );
	gbc.gridx--;
	gbc.gridy++;


	panel.add( new JLabel( GUIIcons.getIcon( GUIIcons.MINI_SEQ_ICON ) ), gbc );
	gbc.gridx++;

	JRadioButton option2 = new JRadioButton("Sequence", false);
	choices.add( option2 );
	// make clicking this button change the type
	option2.addActionListener( new ActionListener(){

		public void actionPerformed( ActionEvent e ){
		    conType = 2;
		}

	    });

	panel.add( option2 , gbc );
	gbc.gridx--;
	gbc.gridy++;


	panel.add( new JLabel( GUIIcons.getIcon( GUIIcons.MINI_ALT_ICON ) ), gbc );
	gbc.gridx++;

	JRadioButton option3 = new JRadioButton("Alternatives", false);
	choices.add( option3 );
	// make clicking this button change the type
	option3.addActionListener( new ActionListener(){

		public void actionPerformed( ActionEvent e ){
		    conType = 3;
		}

	    });

	panel.add( option3 , gbc );
	gbc.gridx--;
	gbc.gridy++;


	//=================== set last user choice =============
	if ( conType != 1 ){
	    
	    switch (conType){

	    case 2:
		option2.doClick();
		break;
		
	    case 3:
		option3.doClick();
		break;
	    }
	}



	prevButton.setEnabled( true );
	return panel;
    }







    //====================================================================================



    /**
       Swaps the currently displayed panel

       @param panel the new panel to replace the old one
     */
    private void swapPanel( JPanel panel ){

	pane.remove( 0 );
	pane.add( panel, 0 );
	pane.validate();

    }



    /**
       Runs a wizard prompting the user for value needed to create a resource.

       @return the new resource or null if the user cancelled.
     */
    public RDFResource addResource(){

	// set the title
	window.setTitle("Add a new resource");

	// run the wizard...
	start = 1;
	screen = 1;
	swapPanel( createAddSubjectPanel() );
	window.show();


	// if the wizard completed al these values should be set...
	if (( res != null ) && ( pred != null ) && ( obj != null )){
	    // add property to resource
	    res.addProperty( pred, obj);
	    // and return it
	    return res;
	}

	// otherwise the user cancelled so...
	return null;
    }


    /**
       Runs a wizard prompting the user for value needed to create a property.
       The property will be added to the given resource.

       @return the new resource or null if the user cancelled.
     */
    public RDFProperty addProperty( RDFResource myRes){

	// set the title
	window.setTitle("Add a new property");

	// run the wizard...
	start = 2;
	screen = 2;
	swapPanel( createAddPredicatePanel() );
	window.show();


	// if the wizard completed al these values should be set...
	if (( pred != null ) && ( obj != null )){
	    // add property to resource
	    myRes.addProperty( pred, obj);

	    // get the RDFProperty out of the resource...
	    RDFProperty[] tmp = myRes.getProperties();
	    RDFProperty output = null;
	    // find the one we just added
	    for ( int i=0; i< tmp.length; i++){

		// kinda crude comparison, but it's the best I can do for now...
		if( ( tmp[i].getPredicate().getType() == pred.getType() ) &&
		    ( tmp[i].getObject().getID().equals( obj.getID() ) ) ){

		    //System.out.println("Aha! Found it!");
		    output = tmp[i];
		    break;
		}

	    }

	    // and return it
	    return output;
	}

	// otherwise the user cancelled so...
	return null;
    }


    /**
       Runs a wizard prompting the user to make an object.

       @return the new object or null if the user cancelled.
     */
    public RDFObject addObject(){

	// set the title
	window.setTitle("Add a new object");

	// run the wizard...
	start = 3;
	screen = 3;
	swapPanel( createAddObjectPanel() );
	window.show();


	// if the wizard completed al these values should be set...
	if ( obj != null ){
	    // and return it
	    return obj;
	}

	// otherwise the user cancelled so...
	return null;
    }



    /**
       Cell renderer for the resources.
	
       @author James Nash
    */
    class ResourceRenderer extends JLabel implements ListCellRenderer{


	/**
	   Constructs a new PropTypeRenderer
	*/
	public ResourceRenderer(){
	    setOpaque( true );

	    int height = (int)  getPreferredSize().getHeight();
	    
	}

	/**
	   Render an item from the list

	   @param list the JList whose cell is being rendered
	   @param val the value of the cell
	   @param row the row of the cell	   
	   @param isSelected if the cell is currently selected
	   @param hasFocus if the cell has focus
	*/
	public Component getListCellRendererComponent(JList list, Object val, int row, boolean isSelected, boolean hasFocus){

	    
	    if ( isSelected ){
		setBackground( list.getSelectionBackground() );
		setForeground( list.getSelectionForeground() );
	    }
	    else{
		setBackground( list.getBackground() );
		setForeground( list.getForeground() );
	    }

	    RDFResource myRes = (RDFResource) val;
	    String subject = myRes.getURI();
	    String type = myRes.getTypeDescr();

	    setIcon( gui.getResList().findResource( myRes ).getIcon() );

	    if ( myRes.isAnonymous() ){

		if ( type == null ){
		    setText("Anonymous");
		    setToolTipText("This is an anonymous resource with ID: "+subject);
		}
		else{
		    setText(type+" (Anonymous)");
		    setToolTipText("This is an anonymous resource of type "+type+" with ID: "+subject);
		}
	    }
	    else{

		if ( type == null ){
		    setText(subject);
		    setToolTipText("This is a resource describing "+subject);
		}
		else{
		    setText(type+" ("+subject+")");
		    setToolTipText("This is a resource of type "+type+" describing "+subject);
		}
	    }

	    // if text is to long shorten it
	    if ( getText().length() > 30 ){
		
		setText( getText().substring(0, 27) + ". . ." );

	    }

	    return this;
	}


    }


    /**
       Cell renderer for the property types.
	
       @author James Nash
    */
    class PropTypeRenderer extends JLabel implements ListCellRenderer{


	/**
	   Constructs a new PropTypeRenderer
	*/
	public PropTypeRenderer(){
	    setOpaque( true );
	}

	/**
	   Render an item from the list

	   @param list the JList whose cell is being rendered
	   @param val the value of the cell
	   @param row the row of the cell	   
	   @param isSelected if the cell is currently selected
	   @param hasFocus if the cell has focus
	*/
	public Component getListCellRendererComponent(JList list, Object val, int row, boolean isSelected, boolean hasFocus){

	    
	    if ( isSelected ){
		setBackground( list.getSelectionBackground() );
		setForeground( list.getSelectionForeground() );
	    }
	    else{
		setBackground( list.getBackground() );
		setForeground( list.getForeground() );
	    }

	    RDFPredicate prop = (RDFPredicate) val;

	    setText( prop.getDescr() );
	    setToolTipText( prop.getType() );

	    // if text is to long shorten it
	    if ( getText().length() > 30 ){
		
		setText( getText().substring(0, 27) + ". . ." );

	    }

	    return this;
	}


    }

}
