/*
    Copyright 2004 James Nash
                                                                                                
    This file is part of Ganesha.
                                                                                                
    Ganesha is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                                
    Ganesha is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
                                                                                                
    You should have received a copy of the GNU General Public License
    along with Ganesha (see the LICENSE file); if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
                                                                                                
*/
package com.twiddles.ganesha.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.util.Vector;
import com.twiddles.ganesha.functionality.*;
import com.twiddles.ganesha.gui.gfx.*;

/**
   A container editing window.
   This is a window that may appear in Ganesha's internal
   desktop. It displays the values of a given container
   and allows the user to add, edit or delete them.

   @author James Nash
*/
class ContainerWindow extends JInternalFrame{

   
    /**
       The table containing the value data
    */
    private JTable jt;

    // the container shown in this window
    private RDFContainer con;

    // The resource list that spawned this window
    private ResourceList resList;

    /**
       Button to add new properties.
    */
    private JButton addButton;
    
    /**
       Button to remove selected properties.
    */
    private JButton removeButton;

    /**
       Button to edit selected properties.
    */
    private JButton editButton;

    /**
       any extra buttons
    */
    private JButton[] extraButtons;

    /**
       Constructs a new container editing window.
       The resource's subject will appear as the window title.

       @param con the container that should be displayed in this window
       @param resList the resource list that spawned this window
    */
    public ContainerWindow( RDFContainer con, ResourceList resList ){

	// make window with title that is resizable, closable, maximisable and iconifiable
	super( "Container", true, true, true, true );
	

        // fix the title...
        Icon icon = null;
        String title = "";
        
        // is it a sequence that will have a 2-column table?
        boolean isSeq = false;

	// is it an alt?
	boolean isAlt = false;
        
	// extra buttons for sequences and alts
	extraButtons = null;

	if ( con instanceof RDFSeq ){
            title = "Sequence";
            icon = GUIIcons.getIcon( GUIIcons.MINI_SEQ_ICON );

	    extraButtons = new JButton[ 2 ];

	    JButton upButton = new JButton( GUIIcons.getIcon( GUIIcons.MOVE_OBJ_UP_ICON ) );
	    upButton.setToolTipText("Move the selected object up one position");
	    upButton.setEnabled( false );
	    upButton.addActionListener( new ActionListener(){

		public void actionPerformed( ActionEvent e ){

		    int[] selection = ContainerWindow.this.jt.getSelectedRows();

		    ContainerTableModel ctm = (ContainerTableModel) ContainerWindow.this.jt.getModel();
		    ctm.moveUp( selection[0] );
		    
		    // update gui
		    ContainerWindow.this.resList.didEdit();
		   
		    clearSelect();
		    
		    // keep the element we moved up selected!
		    int nuSelect = selection[0] - 1;
		    ContainerWindow.this.jt.getSelectionModel().setSelectionInterval( nuSelect, nuSelect);

		}

	    });


	    JButton downButton = new JButton( GUIIcons.getIcon( GUIIcons.MOVE_OBJ_DOWN_ICON ) );
	    downButton.setToolTipText("Move the selected object down one position");
	    downButton.setEnabled( false );
	    downButton.addActionListener( new ActionListener(){

		public void actionPerformed( ActionEvent e ){

		    int[] selection = ContainerWindow.this.jt.getSelectedRows();

		    ContainerTableModel ctm = (ContainerTableModel) ContainerWindow.this.jt.getModel();
		    ctm.moveDown( selection[0] );
		    
		    // update gui
		    ContainerWindow.this.resList.didEdit();
		   
		    clearSelect();
		    
		    // keep the element we moved down selected!
		    int nuSelect = selection[0] + 1;
		    ContainerWindow.this.jt.getSelectionModel().setSelectionInterval( nuSelect, nuSelect);

		}

	    });

	    extraButtons[0] = upButton;
	    extraButtons[1] = downButton;

            isSeq = true;
        }
        else if ( con instanceof RDFBag ){
            title = "Bag";
            icon = GUIIcons.getIcon( GUIIcons.MINI_BAG_ICON );
        }
        else if ( con instanceof RDFAlt ){
            title = "Alternatives";
            icon = GUIIcons.getIcon( GUIIcons.MINI_ALT_ICON );

	    extraButtons = new JButton[ 1 ];

	    JButton defaultButton = new JButton( GUIIcons.getIcon( GUIIcons.MAKE_OBJ_DEF_ICON ) );
	    defaultButton.setToolTipText("Make the selected object the default");
	    defaultButton.setEnabled( false );
	    defaultButton.addActionListener( new ActionListener(){

		public void actionPerformed( ActionEvent e ){

		    int[] selection = ContainerWindow.this.jt.getSelectedRows();

		    ContainerTableModel ctm = (ContainerTableModel) ContainerWindow.this.jt.getModel();
		    ctm.makeDefault( selection[0] );
		    
		    // update gui
		    ContainerWindow.this.resList.didEdit();
		   
		    clearSelect();
		    
		    // keep the that we made default selected!
		    ContainerWindow.this.jt.getSelectionModel().setSelectionInterval( 0, 0);

		}

	    });


	    extraButtons[0] = defaultButton;

	    isAlt = true ;
        }      


	if ( con.isAnonymous() ){

	    title += " (Anonymous ID:"+con.getURI()+")";
	}
	else{

	     title += " (About:"+con.getURI()+")";
	}

        // put the icon in the title bar
        setTitle( title );
	setFrameIcon( icon );


	this.con = con;
	this.resList = resList;


	// set initial size
	// ...do this
	setSize(ResourceWindow.DEFAULT_WIDTH, ResourceWindow.DEFAULT_HEIGHT);

        // add lsitener
        addInternalFrameListener( new ContainerWindowListener(con, resList) );

        // create Buttons
        addButton = new JButton( GUIIcons.getIcon( GUIIcons.ADD_PROPERTY_ICON ) );
        addButton.setToolTipText("Add a new object to this container");

        removeButton = new JButton( GUIIcons.getIcon( GUIIcons.REMOVE_PROPERTY_ICON ) );
        removeButton.setToolTipText("Remove selected objects from this container");

        editButton = new JButton( GUIIcons.getIcon( GUIIcons.EDIT_PROPERTY_ICON ) );
        editButton.setToolTipText("Edit the selected object");



	addButton.addActionListener( new ActionListener(){

		// add a new property!
		public void actionPerformed( ActionEvent e ){

		    // ask user for info to create new resource
		    AddWizard dialog = new AddWizard( ContainerWindow.this.resList.getGUI() );
		    RDFObject myObj = dialog.addObject();
		    
		    if ( myObj != null ){
			// user made a property
			//System.out.println("Gonna add new property!");

			ContainerTableModel ctm = (ContainerTableModel) ContainerWindow.this.jt.getModel();
			ctm.addRow( myObj );

			// update gui
			ContainerWindow.this.resList.didEdit();
		    }
		    // else user cancelled

		}

	    });

	
    
    
        // make the buttons that go under the list
        Box buttons = Box.createHorizontalBox();
        // space on the left...
        buttons.add(Box.createHorizontalGlue());
	// ...the buttons...
        buttons.add(addButton);
        buttons.add(removeButton);
        buttons.add(editButton);

	// add any extra buttons
	if ( extraButtons != null ){
	    buttons.add( Box.createHorizontalStrut( 5 ) );
	    for( int i=0; i< extraButtons.length; i++){

		buttons.add( extraButtons[ i ] );

	    }
	}

	// ...and space on the right so they get centred
        buttons.add(Box.createHorizontalGlue());

	buttons.setBorder( BorderFactory.createEmptyBorder(5,5,5,5) );
        

	// create table!
	jt = new JTable( new ContainerTableModel(con) );

	
        
	// set column width and renderer
	TableColumn col = jt.getColumnModel().getColumn(0);
	
        if ( isSeq ){
            // sequence -> 2 columns
            col.setPreferredWidth(40);
	
            col = jt.getColumnModel().getColumn(1);
            col.setCellRenderer( new ObjectRenderer( resList ) );
            col.setPreferredWidth(140);
            
        }
        else{
            // other container -> 1 column
            col.setCellRenderer( new ObjectRenderer( resList, isAlt ) );
        }
	

	// add a selection listener to enable or disable 
	// the edit and remove buttons
	jt.getSelectionModel().addListSelectionListener( new ValueSelectionListener() );
        
        
	// add a mouselistener for editing rows when they are double clicked
	jt.addMouseListener(new MouseInputAdapter(){
		public void mouseClicked(java.awt.event.MouseEvent e){
		    // double click?
		    if (e.getClickCount() == 2){
			// which row (if any) was it?
			int[] entries = getSelectedEntries();
			if ( entries.length == 1 ){
			    // only interested in one row being double clicked

                            ObjectEditor.editObject( ContainerWindow.this.resList, ContainerWindow.this, jt, entries[0], getObjectColumn() );
			                                
			    // clear selection
			    clearSelect();

                            refresh();
			}
			// else do fuck all! :P
		    }
		}
	    } );


        // call clear selection to disable / enable buttons correctly
        //clearSelect();
        removeButton.setEnabled( false );
	editButton.setEnabled( false );	


	// add listener to edit button
	editButton.addActionListener( new ActionListener(){

		public void actionPerformed( ActionEvent e ){

		    // which row (if any) was it?
		    int[] entries = getSelectedEntries();
		    if ( entries.length == 1 ){
			
			ObjectEditor.editObject( ContainerWindow.this.resList, ContainerWindow.this, jt, entries[0], getObjectColumn() );
			
			// clear selection
			clearSelect();
			
			refresh();
		    }
		    // else do fuck all! :P
		    
		}

	    });


	// add listener to remove button
	removeButton.addActionListener( new ActionListener(){

		public void actionPerformed( ActionEvent e ){

		     int[] selection = getSelectedEntries();
		    // confirm that user wants to delete entries
		    String message = "the selected value?";
		    if ( selection.length > 1 ){
			message = "the "+selection.length+" selected values?";
		    }
		    int confirmation = JOptionPane.showConfirmDialog( ContainerWindow.this, "Are you sure you want to delete\n"+message, "Confirm deleting values", JOptionPane.OK_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE);
		    if ( confirmation == JOptionPane.OK_OPTION ){
			// yes!

			ContainerWindow.this.resList.deleteObjectRows( ((ObjectTable) jt.getModel() ),getSelectedEntries() );
		    }
		    else{
			// otherwise just clear selection
			clearSelect();
		    }
		}

	    });


	// create scroll pane to hold table
	JScrollPane spane = new JScrollPane();
	spane.setViewportView( jt );



	// put some stuff into the window
	JComponent pane = (JComponent) this.getContentPane();

	pane.add( spane, BorderLayout.CENTER );
        pane.add(buttons, BorderLayout.SOUTH);

    }






    /**
       Table model for the objects of this container
       
       @author James Nash
    */
    class ContainerTableModel extends AbstractTableModel implements ObjectTable {
    
	/**
	   The table's data
	*/
	private RDFContainer con;
	
        
	/**
	   Constructs a table model of the container's objects.
	   
	   @param con the container whose objects are listed in this table
	*/
	public ContainerTableModel( RDFContainer con ){

            this.con = con;
        
	}

	/**
	   How many columns our table has.
            
           
	   @return amount of columns in table
	*/
	public int getColumnCount(){
        
            // sequences have two columns, alts and bags have one
            if ( con instanceof RDFSeq ){
                return 2;
            }
            
            return 1;
        }

	/**
	   Amount of rows in table.
	   
	   @return amount of rows in table
	*/
	public int getRowCount() { return con.getSize(); }

	/**
	   Retrieve a column's name.

	   @param col the column whose name you want
	   @return column's name
	*/
	public String getColumnName( int col ){
            String name = "Value";
        
            if (( con instanceof RDFSeq ) && ( col == 0 )){
                    // first column of sequence table is the index
                    name = "Index";
            }
        
	    return name;
	}

	/**
	   Get value of a certain cell.
           Only the row matters in this case!

	   @param row row coordinate
	   @param col col coordinate
	   @return cell's value
	*/
	public Object getValueAt(int row, int col){

            if (( getColumnCount() == 2 ) && ( col == 0 )){
            
                //first column of sequence is index
                Integer index = new Integer( (row+1) );
                return index;
            }

            // else get the object

            // get the object at this position in the container
            // (note that container numbering starts at 1, not 0)
	    return con.getObject( (row+1) );

	}


	/**
	   Set value of a certain cell.
           Only the row matters in this case!

	   @param obj the value to be set in the cell
	   @param row row coordinate
	   @param col col coordinate
	*/
	public void setValueAt(Object obj, int row, int col){

	    // if it is the first column of a sequence table
	    // we don't do anything
            if ( ( getColumnCount() == 1 ) || ( col != 0 ) ){
		// otherwise change the object...

		RDFObject robj = null;

		if ( obj instanceof String ){
		    // literal value
		    robj = ContainerWindow.this.resList.createLiteral( (String) obj );

		}
		else{
		    // must be RDFObject of some kind
		    robj = (RDFObject) obj;

		}

		con.setObjectAt( (row+1), robj);

	    }

	}


        /**
            Get the RDF Object that contains the data for this row.
            
            @param row the row containing the object to retrieve
        */
        public RDFObject getObjectAt(int row){

	    return con.getObject( (row+1) );

        }




	/**
	   Get the type of class used in each column.

	   @param col the column for which to get the class
	   @return that column's class
	*/
	public Class getColumnClass(int col) {
	    return getValueAt(0, col).getClass();
	}
	
	/**
	   Determine if a given cell is editable.
	   In this case none of the cells are!

	   @param row ignored
	   @param col ignored
	   @return always false
	*/
	public boolean isCellEditable(int row, int col) {
	    return false;
	}

	
	/**
	   Adds a new object to the table

	   @param obj the object to add
	*/
	public void addRow(RDFObject obj){

	    con.addObject( obj );

	    int index = con.getSize();
	    // notify any listeners that this has happened
	    fireTableRowsInserted(index, index);

	}


	/**
	   Deletes a given row from the table.

	   @param index the row to delete
	*/
	public void deleteRow(int index){
	    
	    // delete data
	    con.removeObject( (index+1) );
	    
	    // notify any listeners that this has happened
	    fireTableRowsDeleted(index, index);
	}


	/**
	   Makes the selected row the default.
	   (only works if the container is an ALT)

	   @param index the row to make default
	*/
	public void makeDefault(int index){
	    
	    RDFAlt alt = (RDFAlt) con;
	    alt.setDefault( (index+1) );

	    fireTableRowsUpdated( 0, index);


	}

	/**
	   Moves the selected row up in the sequence.
	   (only works if the container is an SEQ)

	   @param index the row to move up
	*/
	public void moveUp(int index){
	    
	    RDFSeq seq = (RDFSeq) con;
	    seq.move( (index+1), index );

	    fireTableRowsUpdated( (index-1), index);


	}

	/**
	   Moves the selected row down in the sequence.
	   (only works if the container is an SEQ)

	   @param index the row to move down
	*/
	public void moveDown(int index){
	    
	    RDFSeq seq = (RDFSeq) con;
	    seq.move( (index+1), (index+2) );

	    fireTableRowsUpdated( index, (index+1) );

	}




    }



    /**
       Refreshes the display of the table.
       Intended for use when resources have been added / deleted and
       contents of the resource window might change as a result.
    */
    public void refresh(){

	jt.repaint();

    }
    
    
    
     /**
       ActionListener to detect user selections in the property list.

       @author James Nash
    */
    class ValueSelectionListener implements ListSelectionListener{

	/**
		Enables or disables the 'remove object' and 'edit object' buttons when rows of the
		value list are selected by the user. If one object is seleced both buttons will be
		enabled, if more than one object is selected only 'remove objects' will be enabled
		and if nothing is selected neither button will be available.

		@param e The type of list selection event that is occurring
	*/
	public void valueChanged(ListSelectionEvent e){

	    // ignore adjustments
	    if (e.getValueIsAdjusting()){
		return;
	    }

	    ListSelectionModel l = (ListSelectionModel)e.getSource();
	    if (l.isSelectionEmpty()) {
		// if no rows are selected
		// the remove and edit buttons should be disabled
		removeButton.setEnabled( false );
		editButton.setEnabled( false );

		// extra buttons too
		if ( extraButtons != null ){
		    for (int i=0; i< extraButtons.length; i++ ){
			extraButtons[i].setEnabled( false );
		    }
		}

	    }
	    else{
		// something is selected
		//
		removeButton.setEnabled( true );
                
                // can only edit one property at a time
                if ( jt.getSelectedRowCount() == 1 ){
                    editButton.setEnabled( true );
		    // extra buttons too
		    if ( con instanceof RDFSeq ){
			// this is a sequence
			int[] selection = jt.getSelectedRows();
			
			// 'up' button (don't enable on first row)
			if ( selection[0] != 0 ){
			    extraButtons[0].setEnabled( true );
			}
			else{
			    extraButtons[0].setEnabled( false );
			}

			// 'down' button (don't enable on last row)
			if ( selection[0] != (jt.getRowCount() - 1) ){
			    extraButtons[1].setEnabled( true );
			}
			else{
			    extraButtons[1].setEnabled( false );
			}

		    }
		    else if ( con instanceof RDFAlt ){
			// this is a set of alternatives
			int[] selection = jt.getSelectedRows();

			// default button ( don't enable if first row (ie default) is selected
			if ( selection[0] != 0 ){
			    extraButtons[0].setEnabled( true );
			}
			else{
			    extraButtons[0].setEnabled( false );
			}
		    }
			

                }
                else{
                    editButton.setEnabled( false );
		    // extra buttons too
		    if ( extraButtons != null ){
			for (int i=0; i< extraButtons.length; i++ ){
			    extraButtons[i].setEnabled( true );
			}
		    }

                }

	    }
	}
    }
    
    
    /**
       Clears any current selection of resources.
    */
    public void clearSelect(){
	
	jt.clearSelection();

    } 


    /**
        Get the URI (or anonymous ID) that this container is describing
    */
    public String getURI(){
        return con.getURI();
    }


    /**
       Returns an array of all indices of entries in the property list that
       are currently selected.

       @return array of selected indices
    */
    public int[] getSelectedEntries(){

	return jt.getSelectedRows();

    }

    /**
       Returns the index of the table column that holds the objects

       @return column index
    */
    public int getObjectColumn(){

	return (jt.getColumnCount() - 1);

    }


}
