/*
    Copyright 2004 James Nash
                                                                                                
    This file is part of Ganesha.
                                                                                                
    Ganesha is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                                
    Ganesha is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
                                                                                                
    You should have received a copy of the GNU General Public License
    along with Ganesha (see the LICENSE file); if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
                                                                                                
*/
package com.twiddles.ganesha.gui;

import javax.swing.*;
import java.awt.event.*;
import java.util.*;
import  com.twiddles.ganesha.gui.gfx.*;
/**
   The menubar for the main app window.
   The menubar that runs along the top of the application window,
   it's submenus, it's options and functionality are all defined
   by this class.
   
   @author James Nash
*/
class GMenu extends JMenuBar{

    /**
       Reference to the main app window that contains this menu.
    */
    private GUI window;

    /**
       The look and feel that the Ganesha app was started with.
    */
    private String initialSkin;

    /**
       Contains menu items that need to be enabled/disabled
       when the current file has been edited.

       @see #editedUpdate(boolean)
    */
    private Vector changeOnEdit;

    /**
       Contains menu items that need to be enabled/disabled
       when a file is loaded into the GUI.

       @see #loadedUpdate(boolean)
    */
    private Vector changeOnLoad;

    /**
       A reference to the 'Save' item in the 'File' menu
    */
    private JMenuItem saveCommand;

    /**
       Should the save item be enabled?
    */
    private boolean saveEnabled;

    /**
       Constructs a new Ganesha menu bar.

       @param window the main app window that will contain the menu
       @param initialSkin the Look an feel that was in use when the Ganesha app started
    */
    public GMenu( GUI window, String initialSkin ){

	this.window = window;
	this.initialSkin = initialSkin;

	this.changeOnLoad = new Vector();
	this.changeOnEdit = new Vector();
        
        this.saveCommand = null;
        this.saveEnabled = true;

	// create menu....
	JMenu menu = fileMenu();
	menu.setMnemonic(KeyEvent.VK_F);
	menu.setDisplayedMnemonicIndex(0);
	this.add( menu );

	menu = editMenu();
	menu.setMnemonic(KeyEvent.VK_E);
	menu.setDisplayedMnemonicIndex(0);
	this.add( menu );

	menu = windowMenu();
	menu.setMnemonic(KeyEvent.VK_W);
	menu.setDisplayedMnemonicIndex(0);
	this.add( menu );

	menu = helpMenu();
	menu.setMnemonic(KeyEvent.VK_H);
	menu.setDisplayedMnemonicIndex(0);
	this.add( menu );

    }


    /**
       Generate a menu containing all the skins the user can choose from.
       The menu is presented with radio buttons beside the menu items since
       only one can be selected at a time. Initially the skin passed to the
       constructor will be selected.

       @see GMenu(GUI, String)
       @see SkinEvent
    */
    private JMenu skinMenu(){

	JMenu menu = new JMenu("Switch skin to");

	JRadioButtonMenuItem item;
	ButtonGroup skins = new ButtonGroup();



	// get list of (seemingly) available skins
	UIManager.LookAndFeelInfo[] skinList = UIManager.getInstalledLookAndFeels();


	for (int i = 0; i < skinList.length; i++ ){

	    // get short and long names
	    String skinName = skinList[i].getName();
	    String skinClassName = skinList[i].getClassName();

	    // debug
	    //System.out.println( skinName );

	    // check if we can successfully switch to this skin
	    try{
		 UIManager.setLookAndFeel( skinClassName );
	    }
	    catch( Exception e ){
		// nope, then don't add it to the menu
		continue;
	    }
	    // skin is usable..

	    // switch back to previous skin before we actually make a menu item
	    try {
		UIManager.setLookAndFeel( initialSkin );
	    }
	    catch( Exception e ){
		// this should never happen since we just checked we could use the skin!
	    }
	    

	    // is it the one we were using before?
	    if ( skinClassName == initialSkin ){
		item = new JRadioButtonMenuItem( skinName, true );
	    }
	    else{
		item = new JRadioButtonMenuItem( skinName );
	    }
	    item.addActionListener( new SkinEvent( window, skinClassName) );

	    skins.add(item);
	    menu.add(item);


	}



	return menu;
    }


    
    /**
       Enable or disable menu items that are effected by a file (not) have being loaded.

       @param loaded whether or not a file is currently loaded
       @see GUI#setLoaded(boolean)
    */
    public void loadedUpdate( boolean loaded ){
	// temporary placeholder
	JMenuItem tmp;

	// enable or disable all relevant menu items
	for (int i=0; i < changeOnLoad.size(); i++){
	    tmp = (JMenuItem) changeOnLoad.elementAt(i);
	    tmp.setEnabled( loaded );
	}

    }


    /**
       Enable or disable menu items that are effected by the file (not) have being changed.

       @param edited whether or not the loaded file has been edited
       @see GUI#setEdited(boolean)
    */
    public void editedUpdate( boolean edited ){
	// temporary placeholder
	JMenuItem tmp;

	// enable or disable all relevant menu items
	for (int i=0; i < changeOnEdit.size(); i++){
	    tmp = (JMenuItem) changeOnEdit.elementAt(i);
	    tmp.setEnabled( edited );
	}
        
        // save is a related menu item
        saveCommand.setEnabled( saveEnabled && edited );
        
    }


    /**
        Enable or disable saving.
        Use to prevent the user trying to save read-only files.
        ( 'save as' is unaffected by this )

	@param enabled should the 'Save' option be enabled or not
	@see GUI#setSaveEnable(boolean)
    */
    public void setSaveEnable( boolean enabled ){

        saveEnabled = enabled;
    
    }





    /**
       Builds the 'File' menu.
       This menu contains file related operations like open, save, close etc.
              
       @return the complete, fully-functional 'File' menu
       @see GUIEvent
    */
    private JMenu fileMenu(){
	JMenu fileMenu = new JMenu("File");

	JMenuItem item = new JMenuItem("New", GUIIcons.getIcon( GUIIcons.MINI_NEW_ICON ) );
	item.setAccelerator( KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_MASK) );
	item.setMnemonic(KeyEvent.VK_N);
	item.setDisplayedMnemonicIndex(0);
	item.addActionListener(new GUIEvent(window, GUIEvent.NEW_FILE));
	fileMenu.add(item);

	item = new JMenuItem("Open...", GUIIcons.getIcon( GUIIcons.MINI_OPEN_ICON ) );
	item.setAccelerator( KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_MASK) );
	item.setMnemonic(KeyEvent.VK_O);
	item.setDisplayedMnemonicIndex(0);
	item.addActionListener(new GUIEvent(window, GUIEvent.OPEN_FILE));
	fileMenu.add(item);

	item = new JMenuItem("Close");
	item.setAccelerator( KeyStroke.getKeyStroke(KeyEvent.VK_W, InputEvent.CTRL_MASK) );
	item.setMnemonic(KeyEvent.VK_C);
	item.setDisplayedMnemonicIndex(0);
	item.addActionListener(new GUIEvent(window, GUIEvent.CLOSE_FILE));
	// can only close if a file is loaded
	changeOnLoad.add(item);
	fileMenu.add(item);

	fileMenu.addSeparator();

	item = new JMenuItem("Save", GUIIcons.getIcon( GUIIcons.MINI_SAVE_ICON ) );
	item.setAccelerator( KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK) );
	item.setMnemonic(KeyEvent.VK_S);
	item.setDisplayedMnemonicIndex(0);
	item.addActionListener(new GUIEvent(window, GUIEvent.SAVE_FILE));
	// can only saving only works if it is enabled AND modifications to the current file have been made
        saveCommand = item;
        fileMenu.add(item);

	item = new JMenuItem("Save As...", GUIIcons.getIcon( GUIIcons.MINI_SAVE_AS_ICON ) );
	item.setAccelerator( KeyStroke.getKeyStroke(KeyEvent.VK_S, (InputEvent.CTRL_MASK +InputEvent.ALT_MASK ) ) );
	item.setMnemonic(KeyEvent.VK_A);
	item.setDisplayedMnemonicIndex(1);
	item.addActionListener(new GUIEvent(window, GUIEvent.SAVE_FILE_AS));
	// can only save as if a file is loaded
	changeOnLoad.add(item);	
	fileMenu.add(item);

	fileMenu.addSeparator();

	item = new JMenuItem("Exit");
	item.setMnemonic(KeyEvent.VK_X);
	item.setDisplayedMnemonicIndex(1);
	item.addActionListener(new GUIEvent(window, GUIEvent.SHUTDOWN));
	fileMenu.add(item);

	return fileMenu;
    }



    /**
       Builds the 'Edit' menu.
       This menu contains editing operations such as adding or removing resources.
       
       @return the complete, fully-functional 'Edit' menu
       @see GUIEvent
    */
    private JMenu editMenu(){
	JMenu editMenu = new JMenu("Edit");

	JMenuItem item = new JMenuItem("Add this RDF to a webpage...");
	item.setMnemonic(KeyEvent.VK_A);
	item.setDisplayedMnemonicIndex(0);
	item.addActionListener(new GUIEvent(window, GUIEvent.HTML_LINK));
	// can only edit stuff if a file is loaded
	changeOnLoad.add(item);
	editMenu.add(item);


	return editMenu;
    }


    /**
       Builds the 'Window' menu.
       This menu contains operations for controlling the appearance of windows
       such as changing the skin, or organising the internal windows.
              
       @return the complete, fully-functional 'Window' menu
       @see GUIEvent
    */
    private JMenu windowMenu(){
	JMenu windowMenu = new JMenu("Window");

	JMenu item = skinMenu();
	item.setMnemonic(KeyEvent.VK_S);
	item.setDisplayedMnemonicIndex(0);
	windowMenu.add(item);

	return windowMenu; 
    }

    


    /**
       Builds the 'Help' menu.
       This menu lets users call up information about the program
       and help files.
              
       @return the complete, fully-functional 'Help' menu
       @see GUIEvent
    */
    private JMenu helpMenu(){
	JMenu helpMenu = new JMenu("Help");

	JMenuItem item = new JMenuItem("About");
	item.setMnemonic(KeyEvent.VK_A);
	item.setDisplayedMnemonicIndex(0);
	item.addActionListener(new GUIEvent(window, GUIEvent.ABOUT_SCREEN));
	helpMenu.add(item);

	return helpMenu; 
    }


}
