/*
    Copyright 2004 James Nash
                                                                                                
    This file is part of Ganesha.
                                                                                                
    Ganesha is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                                
    Ganesha is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
                                                                                                
    You should have received a copy of the GNU General Public License
    along with Ganesha (see the LICENSE file); if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
                                                                                                
*/
package com.twiddles.ganesha.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import com.twiddles.ganesha.functionality.*;
import com.twiddles.ganesha.Settings;
import com.twiddles.ganesha.gui.gfx.GUIIcons;

/**
   The main application window of Ganesha.
   Once instanciated it creates the various GUI components that make up
   the application window and maintains references to them to facilitate
   communication between the various GUI elements.
   This class also keeps track of various states relevant to the GUI
   (such as whether current work has been saved or not).

   @author James Nash
*/
public class GUI extends JFrame{


    

    /**
       true if a file is currently loaded in the GUI
     */
    private boolean loaded;

    /**
       true if a loaded file has been modified since the last save (meaningless if no file is loaded)
    */
    private boolean edited;

    /**
       false if file saving operations are not permitted (like when working with read-only files)
    */
    private boolean saveEnabled;
    
    /**
       name of the Java Look and Feel that is currently in use
    */
    private String skin;

    /**
       RDF contents of a currently loaded file
    */
    private RDFFile rdfFile;


    // main components of the GUI
    /**
       reference to the GUI's toolbar
    */
    private GToolbar toolbar;
    /**
       reference to the GUI's menu bar
    */
    private GMenu menu;
    /**
       reference to the GUI's main working area, where the resource editing windows appear
    */
    private GDesktop desktop;
    /**
       reference to the Resource list that runs down the left-hand side of the GUI
    */
    private ResourceList resList;


    /**
       Initialises the Ganesha application window.
       This method will create the main application window and populate it with the
       main GUI elements: the menubar, the toolbar, the resource list and space for
       resource editing windows.

       @see GMenu
       @see GToolbar
       @see ResourceList
       @see GDesktop
    */
    public GUI(){

	// initialise system status
	loaded = false;
	edited = false;
	saveEnabled = true;
        
	// initialise the RDFfile to be used
	//
	// Currently we are using the Jena implementaion,
	// but if that changed this should be the only line
	// that needs changing!
	rdfFile = new JenaFile();
	

	// set Look and Feel (default one for now)
	skin = UIManager.getSystemLookAndFeelClassName();
	switchLNF( skin );
	

	// perpare components
	toolbar = new GToolbar(this);
	menu = new GMenu(this, skin);
	desktop = new GDesktop(this);
	resList = new ResourceList(this, desktop);


	// prepare window
	setIconImage( GUIIcons.getIcon( GUIIcons.MINI_GANESHA_ICON ).getImage() );
	updateTitle();
	setSize(800, 600);

	// add menubar
	setJMenuBar( menu );
	// add toolbar
	Container contentPane = getContentPane();
	contentPane.add( toolbar, BorderLayout.NORTH);
	

	// add resource list and desktop
	JSplitPane panes = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, true, resList, desktop);
	panes.setDividerLocation( 200 );

	contentPane.add ( panes, BorderLayout.CENTER);


	
	// stuff to be done when window is closed by user...
	this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
	WindowListener listener = new GUIWindowListener(this);
	this.addWindowListener(listener);

	// finally set enable / disable menus correctly
	updateGUI();

	
    }


    // ####################################################################
    // #                      Private methods                             #
    // ####################################################################
    
    /**
       Enables or disables ('greys out') buttons, menus and the window title based on
       the GUI's current status.
       Instructs the menu and toolbar to update themselves based on whether a file is loaded
       and if it has been modified. (For example - the 'Save' option will be disabled if no
       file is loaded). Mostly there is no need to call this method directly. Instead use
       one of the following:

       @see #setEdited(boolean)
       @see #setLoaded(boolean)
       @see #setSaveEnable(boolean)
    */
    private void updateGUI(){
        // set saveEnable flag in menu
        menu.setSaveEnable( saveEnabled );
    
	// update menu items
	menu.loadedUpdate( loaded );
	menu.editedUpdate( edited );

	// update toolbar buttons too
        toolbar.setSaveEnable( saveEnabled );
	toolbar.loadedUpdate( loaded );
	toolbar.editedUpdate( edited );

           
	// other updates
	updateTitle();
    }

    /**
       Changes the title of the app window based on the GUI's current status.
       For example, if a file is loaded the file's name is added to the
       window title text. This method should NOT be called directly - it is
       intended to be used by <code>updateGUI()</code>.

       @see #updateGUI()
    */
    private void updateTitle(){
	String title = "";

	// is a file loaded?
	if ( loaded ){

	    // is it a file with a name?
	    if ( rdfFile.getFileName() != null ){
		// then append the filename
		title += rdfFile.getFileName();
	    }
	    // or a new, as yet un-named file?
	    else{
		title += "(New file)";
	    }

            // is it read-only?
            if ( !saveEnabled ){
                title += " (read only)";
            }

	    //and has it been modified?
	    if ( edited ){
		//then add a star
		title += "*";
	    }

	    title += " - ";
	}

	title += "Ganesha";

	// set the window title
	setTitle( title );
    }




    // ####################################################################
    // #                       Public methods                             #
    // ####################################################################



    /**
       Returns a reference to the underlying RDF file.
       
       @return reference to the rdf data
    */
    public RDFFile getRDF(){
	return rdfFile;
    }


    /**
       Returns the current resource list
       
       @return reference to the resource list
    */
    public ResourceList getResList(){
	return resList;
    }


    /**
       Switch the GUI's Look and Feel.
       This method will make all GUI elements switch their Java Swing Look and
       Feel to that specified as the parameter.

       @param lnfName the class name of the Java Swing Look and Feel you wish to switch to
    */
    public void switchLNF(String lnfName){
	try {
	    UIManager.setLookAndFeel( lnfName );
	}
	catch( Exception e ){
	    System.err.println("Could not switch skin to "+lnfName+":");
	    System.err.println( e.toString() );
	}
	skin = lnfName;
	SwingUtilities.updateComponentTreeUI(this);
	
	// this squishes window, see if u can get it to pack everything in the
	// window but not the window itself
	//this.pack();

    }



    /**
       Set the file loaded state of the GUI.
       If a file has been (un)loaded in the GUI this method should be
       called to update the GUI's state accordingly. This method also
       makes a call to <code>updateGUI()</code> to make any necessary
       changes to the GUI.

       @param loaded true: file is loaded; false: no file loaded
    */
    public void setLoaded(boolean loaded){
	this.loaded = loaded;
	// when a new file is loaded or an old one is unloaded
	// it cannot have been edited
	this.edited = false;

	// update menus and toolbar
	updateGUI();
    }


    /**
       Set whether or not a loaded file was edited since the last save.
       If a file has been modified in the GUI this method should be
       called to update the GUI's state accordingly. This method also
       makes a call to <code>updateGUI()</code> to make any necessary
       changes to the GUI.

       @param edited true: file is modified; false: not modified since last save
    */    
    public void setEdited(boolean edited){
	this.edited = edited;

	// update menus and toolbar
	updateGUI();
    }


    /**
       Enable/disable save operations in the GUI.
       Use this method if save operations should be disabled in the GUI.
       For example if a read-only file is loaded. This method also
       makes a call to <code>updateGUI()</code> to make any necessary
       changes to the GUI.

       @param enabled enable/disable saving in the GUI
    */ 
    public void setSaveEnable(boolean enabled){
        this.saveEnabled = enabled;
        
        updateGUI();
    }




    /**
       Shutdown the Ganesha application.
       If a file is loaded and has been modified the user will
       be prompted to save his work. Upon saving the application
       will quit.

       @see #saveFile()
     */
    public void shutdown(){
	
	// really shutdown?
	boolean really_exit = true;

	// check if a file is present that needs saving
	if ( loaded && edited ){
	    // yep
	    JOptionPane confirm = new JOptionPane();
	    int choice = confirm.showConfirmDialog(this, "You have not saved your work!\nWould you like to save it before you exit?", "Confirm exit", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE);

	    if ( choice == JOptionPane.YES_OPTION ){
		// save first
		saveFile();

	    }
	    else if ( choice != JOptionPane.NO_OPTION ){
		// they didn't say no so they must have cancelled
		really_exit = false;
	    }
	    // if they said no we just go ahead an quit without saving!
	}


	// are we still sure we want to quit?
	if ( really_exit ){

	    // close GUI
	    this.dispose();
	    // close app
	    System.exit(0);
	}
	// else do fuck all
    }



    /**
       Close a currently loaded file in the GUI.
       If the file has been modified the user will be prompted to save his work.
       Upon saving the file will be closed and the GUI's state and appearance updated
       accordingly.

       @see #saveFile()
    */
    public void closeFile(){

	// really close file?
	boolean really_close = true;

	// check if a file needs saving
	if ( edited ){
	    // yep
	    JOptionPane confirm = new JOptionPane();
	    int choice = confirm.showConfirmDialog(this, "You have not saved your work!\nWould you like to save it before you close it?", "Confirm close", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE);

	    if ( choice == JOptionPane.YES_OPTION ){
		// save first
		saveFile();

	    }
	    else if ( choice != JOptionPane.NO_OPTION ){
		// they didn't say no so they must have cancelled
		really_close = false;
	    }
	    // if they said no we just go ahead an quit without saving!
	}


	// are we still sure we want to quit?
	if ( really_close ){

	    // remove old resource listing
	    resList.showBlank();

	    // file is not loaded
	    setLoaded(false);

	}
	// else do fuck all

    }

    /**
       Open a specified file in the GUI.
       If the file cannot
       be opened (if it is not valid RDF/XML or does not have read
       permission set) the user will be notified. If the file is read-only
       it will be loaded, but saving will be disabled.

       @param file the file to open
    */
    public void openFile(File file){
	
	// make sure we can read the file
	if ( ! file.canRead() ){
	    // if not we can't open!
	    JOptionPane.showMessageDialog(this, "You do not have permission to read '"+file.getName()+"'.\nIt cannot be opened.", "Opening the file failed", JOptionPane.ERROR_MESSAGE );
	}
	else{
	    // load the file
	    try{
		// check file is writeable...
		if ( ! file.canWrite() ){
		    // warn user they will not be able to save
		    JOptionPane.showMessageDialog(this, "You do not have permission to overwrite '"+file.getName()+"'.\nYou will need to save any work you do to a new file using 'Save As'.", "No write permission", JOptionPane.WARNING_MESSAGE );
		    
		    // disable save command
		    setSaveEnable( false );
		}
		else{
		    // we can save!
		    setSaveEnable( true );
		}
                
		rdfFile.loadFile( file );
		    
                   
		// list the resources
		resList.showFile( rdfFile );
		    
		// a file has been loaded
		setLoaded(true);
    
		// remember where we gto this file from
		Settings.setLastOpenDir( file );

	    }
	    catch (RdfIOException e){
		// alert user if it didn't work
		String errMes;
		
		// create relevant error message
		switch ( e.getError() ){
                    
		case RdfIOException.INVALID_RDF:
		    errMes = "'"+file.getName()+"' is not a valid RDF/XML file and cannot be opened.";
		    break;
			    
		case RdfIOException.FILE_NOT_FOUND:
		    errMes = "'"+file.getName()+"' cannot be opened because it does not exist!";
		    break;
		    
		default:
		    errMes = "'"+file.getName()+"' could not be opened for some bizarre reason.\nSorry!";
		    break;
			    
		}
                    
		JOptionPane.showMessageDialog(this, errMes, "Opening the file failed", JOptionPane.ERROR_MESSAGE );
                    
		
	    }
	}

    }


    /**
       Open a file in the GUI.
       This will present the user with a file choosing dialog to pick a
       RDF/XML file to open. The selected file will be read in and
       stored internally as a <code>RDFFile</code>
       If there is currently another file loaded and it has been edited
       the user will be prompted to save his work.

       @see #openFile(File)
       @see #saveFile()
       @see #setSaveEnable(boolean)
       @see com.twiddles.ganesha.functionality.RDFFile
    */
    public void openFile(){

	// if there is a file already loaded AND it has been
	// edited we should check to see if the user wants to save first
	boolean really_open = true;

	if ( loaded && edited ){
	    // yep
	    JOptionPane confirm = new JOptionPane();
	    int choice = confirm.showConfirmDialog(this, "You have not saved your current work\nWould you like to save it before opening something else?", "Confirm opening other file", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE);
	    
	    if ( choice == JOptionPane.YES_OPTION ){
		// save first
		saveFile();

	    }
	    else if ( choice != JOptionPane.NO_OPTION ){
		// they didn't say no so they must have cancelled
		really_open = false;
	    }
	    // if they said no we just go ahead an quit without saving!
	}    
	    

	// no we know if we should open something or not
	if ( really_open ){
	    // prompt for filename
	    JFileChooser fileDialogue = new JFileChooser( Settings.getLastOpenDir() );
            // only show rdf files..
            RDFFileFilter filter = new RDFFileFilter();
            fileDialogue.addChoosableFileFilter( filter );
            fileDialogue.setFileFilter( filter );
            
	    int choice = fileDialogue.showOpenDialog(this);
	
	    // did the user pick something or cancel?
	    if ( choice == JFileChooser.APPROVE_OPTION ){
		File file = fileDialogue.getSelectedFile();

		// go on and open it!
		openFile(file);
               
	    }
	    // if the user cancelled we do nothing
	}
    }





    /**
       Start a blank new file.
       If a modified file is currently loaded the user will be prompted to
       save his work first. If saving was previously disabled it will be
       enabled.

       @see #saveFile()
    */
    public void newFile(){

	// if there is a file already loaded AND it has been
	// edited we should check to see if the user wants to save first
	boolean really_new = true;

	if ( loaded && edited ){
	    // yep
	    JOptionPane confirm = new JOptionPane();
	    int choice = confirm.showConfirmDialog(this, "You have not saved your current work\nWould you like to save it before opening something else?", "Confirm opening other file", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE);
	    
	    if ( choice == JOptionPane.YES_OPTION ){
		// save first
		saveFile();

	    }
	    else if ( choice != JOptionPane.NO_OPTION ){
		// they didn't say no so they must have cancelled
		really_new = false;
	    }
	    // if they said no we just go ahead an quit without saving!
	}    
	    

	// no we know if we should open something or not
	if ( really_new ){
	
	    // go ahead and make new file then
	    rdfFile.newFile();

            // we can save
            setSaveEnable( true );

	 
	    // show the resources...
	    resList.showFile( rdfFile );

	    // we now have a file loaded
	    setLoaded(true);	

	}
	// else we do nothing...
	
    }




    /**
      Save the currently loaded file under its current filename.
      If the file is a new one that does not yet have a name
      <code>saveFileAs()</code> will get called.

      @see #saveFileAs()
    */
    public void saveFile(){

	// If file has no name yet (coz it's a new file)
        // OR is saving is disabled (coz the file is readonly) 
        // we should use saveAs instead!
	if ( (! saveEnabled ) || (rdfFile.getFileName() == null  ) ){
	    // it's a new file so use Save As...
	    saveFileAs();
	}
	else{
	    // otherwise go ahead and save under the current name...

	    try{
		rdfFile.saveFile();
		
		// if the file has just been saved then it can't have been modified
		setEdited(false);
			
	    }
	    catch (RdfIOException e){
		// alert user if it didn't work
		String errMes;

		switch ( e.getError() ){

		case RdfIOException.NO_WRITE_PERM:
		     errMes = "'"+rdfFile.getFileName()+"' could not be saved. Perhaps you do not\nhave write permissions for that folder.";
		     break;
		    
		default:
		    errMes = "'"+rdfFile.getFileName()+"' could not be saved for some bizarre reason.\nSorry!";
		    break;

		}


		JOptionPane.showMessageDialog(this, errMes, "Saving the file failed", JOptionPane.ERROR_MESSAGE );

	    }
	}
	    	   
    }



    /**
       Save a file under a new filename.
       The user will be prompted to choose a new filename to save their work under.
       Then the file gets saved, saving is enabled (incase it was disabled before)
       and the GUI is updated accordingly.
    */
    public void saveFileAs(){
	// we can always save something as something
	// (uness "save as" is disabled in the GUI, but then
	// this method wouldn't get called, would it! :P )

	// ask for a filename
	JFileChooser fileDialogue = new JFileChooser( Settings.getLastSaveAsDir() );
        // only show rdf files..
        RDFFileFilter filter = new RDFFileFilter();
        fileDialogue.addChoosableFileFilter( filter );
        fileDialogue.setFileFilter( filter );
        
        
	int choice = fileDialogue.showSaveDialog(this);
	
	// did the user pick something or cancel?
	if ( choice == JFileChooser.APPROVE_OPTION ){
	    File file = fileDialogue.getSelectedFile();
	    
	    // if the file exists, ask if it is ok to overwrite it!
	    boolean really_save = false;
	    if ( file.exists() ){

		int overwrite = JOptionPane.showConfirmDialog( this, "The file "+file.getName()+" already exists. Are you sure you want to overwrite this file?", "Confirm overwrite old file", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE);

		if ( overwrite == JOptionPane.YES_OPTION ){
		    really_save = true;
		}


	    }
	    else{
		// no fiel, so go ahead and save
		really_save = true;
	    }


	    if ( really_save ){
		// save the file under the new name
		try{
		    rdfFile.saveFile( file );
		    
		    // if the file has just been saved then it can't have been modified
		    setEdited(false);
		    // if we have saved as something then saving must be enabled from now on
		    setSaveEnable(true);
		    
		    // remember this directory
		    Settings.setLastSaveAsDir( file );
		    
		}
		catch (RdfIOException e){
		    // alert user if it didn't work
		    // alert user if it didn't work
		    String errMes;
		    
		    switch ( e.getError() ){
			
		    case RdfIOException.NO_WRITE_PERM:
			errMes = "'"+file.getName()+"' could not be saved. Perhaps you do not\nhave write permissions for that folder.";
			break;
			
		    default:
			errMes = "'"+file.getName()+"' could not be saved for some bizarre reason.\nSorry!";
			break;
			
		    }
		
		    JOptionPane.showMessageDialog(this, errMes, "Saving the file failed", JOptionPane.ERROR_MESSAGE );
		}
	    }
	    // if we shouldn't save do nothing

	}
	// if user cancelled we do nothing
    }


}


