/*
    Copyright 2004 James Nash
                                                                                                
    This file is part of Ganesha.
                                                                                                
    Ganesha is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                                
    Ganesha is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
                                                                                                
    You should have received a copy of the GNU General Public License
    along with Ganesha (see the LICENSE file); if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
                                                                                                
*/
package com.twiddles.ganesha.gui;

import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;
import com.twiddles.ganesha.functionality.*;
import com.twiddles.ganesha.gui.gfx.*;
/**
    Renderer for the value (ie object) column
    
    @author James Nash
*/
class ObjectRenderer extends JLabel implements TableCellRenderer{

    // reference to the resource list
    private ResourceList resList;

    // is this an alt?
    private boolean isAlt;

    /**
        Constructs a new ObjectRenderer.

	@param resList the current resource list
    */
    public ObjectRenderer(ResourceList resList) {
        this.resList = resList;
        setOpaque(true);
        setFont( getFont().deriveFont( java.awt.Font.PLAIN ) );
	isAlt = false;
    }

    /**
        Constructs a new ObjectRenderer for alt containers.

	@param resList the current resource list
	@param isAlt whether or not this is an alt
    */
    public ObjectRenderer(ResourceList resList, boolean isAlt) {
        this.resList = resList;
        setOpaque(true);
        setFont( getFont().deriveFont( java.awt.Font.PLAIN ) );
	this.isAlt = isAlt;
    }


    /**
        Returns a JLabel depicting the object's value.
        The label will consist of an icon indicating
        the type of object (literal value, resource or container)
        and the actual value.
        
        @param table the table whose cell is being rendered
        @param obj the predicate object
        @param isSelected whether or not this cell is selected
        @param hasFocus ignored
        @param row ignored
        @param col ignored
        @return a rendered cell
    */
    public Component getTableCellRendererComponent( JTable table, Object obj, boolean isSelected, boolean hasFocus, int row, int col){
        
        RDFObject myObj = (RDFObject) obj;
        String val = "Unsupported value type!";
        String tip = "Sorry, but Ganesha does not understand this value!";
        int type = myObj.getObjType();
        Icon icon = GUIIcons.getIcon( GUIIcons.MINI_UNKNOWN_ICON );

        if ( type == RDFObject.LITERAL ){

            val = myObj.getLiteral();
            tip = "Literal Value: "+val;
            icon = GUIIcons.getIcon( GUIIcons.MINI_LITERAL_ICON );

        }
        else if( type == RDFObject.RESOURCE ){

            icon = ResIconFactory.getGreyIcon();

            // check if it is a resource described in the current file.
            ResourceListEntry tmp = resList.findResource( myObj.getResource() );

            if ( tmp != null ){
                icon = tmp.getIcon();
            }

	    String resType = myObj.getResource().getTypeDescr();

            if ( myObj.getResource().isAnonymous() ){

		if ( resType == null ){
		    val = "Anonymous resource (ID: "+myObj.getResource().getURI()+")";
		}
		else{
		    val = resType+" (Anonymous ID: "+myObj.getResource().getURI()+")";
		}

            }
            else{

		if ( resType == null ){
		    val = myObj.getResource().getURI();
		}
		else{
		    val = resType+" ("+myObj.getResource().getURI()+")";
		}

            }

            tip = "Resource: "+val;
            
        }
        else if( type == RDFObject.BAG ){

            if ( myObj.getBag().isAnonymous() ){
                val = "Bag (Anonymous ID: "+myObj.getBag().getURI()+")";
                tip = val;
            }
            else{
                val = "Bag about: "+myObj.getBag().getURI();
                tip = val;
            }
            icon = GUIIcons.getIcon( GUIIcons.MINI_BAG_ICON );
            
        }
        else if( type == RDFObject.SEQ ){

            if ( myObj.getSeq().isAnonymous() ){
                val = "Sequence (Anonymous ID: "+myObj.getSeq().getURI()+")";
                tip = val;
            }
            else{
                val = "Sequence about: "+myObj.getSeq().getURI();
                tip = val;
            }
            icon = GUIIcons.getIcon( GUIIcons.MINI_SEQ_ICON );
            
        }
        else if( type == RDFObject.ALT ){

            if ( myObj.getAlt().isAnonymous() ){
                val = "Alternatives (Anonymous ID: "+myObj.getAlt().getURI()+")";
                tip = val;
            }
            else{
                val = "Alternatives about: "+myObj.getAlt().getURI();
                tip = val;
            }
            icon = GUIIcons.getIcon( GUIIcons.MINI_ALT_ICON );
            
        }

        // set the colours according to whether it is selected or not
	if (( isAlt ) && ( row == 0)){
	    // default in alt container!
	    if( isSelected ){
		setBackground( table.getSelectionBackground() );
		setForeground( table.getSelectionForeground() );
	    }
	    else{
		// highlight default alt element!
		setBackground( new Color( 151, 242, 178) );
		setForeground( Color.BLACK );
	    }

	    setToolTipText( "[Default choice] "+tip );
	}
	else{
	    if( isSelected ){
		setBackground( table.getSelectionBackground() );
		setForeground( table.getSelectionForeground() );
	    }
	    else{
		setBackground( table.getBackground() );
		setForeground( table.getForeground() );
	    }

	    setToolTipText( tip );
	}
        
        setIcon( icon );
        setText( val );
        
                
        return this;
    }
}
