/*
    Copyright 2004 James Nash
                                                                                                
    This file is part of Ganesha.
                                                                                                
    Ganesha is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                                
    Ganesha is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
                                                                                                
    You should have received a copy of the GNU General Public License
    along with Ganesha (see the LICENSE file); if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
                                                                                                
*/
package com.twiddles.ganesha.gui;

import java.io.*;

/**
    RDF/XML file filter for the file choosing dialogs.
    Filters out any non-rdf files based on their file extensions.

    @author James Nash
*/
class RDFFileFilter extends javax.swing.filechooser.FileFilter{

    /**
       List of permitted RDF/XML filename extensions.
       The extensions are provided in lowercase and without
       the preceding dot.
    */
    private final String[] rdfExtensions = {"rdf", "xrdf"};
    

    /**
        Extract a file's extension.        
        The code for this method is based on an example from the
        "Java(tm) tutorial, How to Use File Choosers", available at:
        <a href="http://java.sun.com/docs/books/tutorial/uiswing/components/filechooser.html#filters">http://java.sun.com/docs/books/tutorial/uiswing/components/filechooser.html#filters</a>
	<p>
	Everything after the first dot in the filename is considered the extension.
        
	@param file the file whose filename extension is to be extracted
	@return the file's extension converted to lowercase; null if there is no extension
    */
    private String getExtension(File file){
        String ext = null;
        String s = file.getName();
        int i = s.lastIndexOf('.');

        if (i > 0 &&  i < s.length() - 1) {
            ext = s.substring(i+1).toLowerCase();
        }
        return ext;
  
    }


    /**
       Filters out files without valid RDF/XML extensions.
       
       @param file a file to be filtered
       @return true if the file has an RDF/XML extension or is a directory; false otherwise
    */        
    public boolean accept( File file ){
        if ( file.isDirectory() ){
            // need to allow dirs, so user can navigate filesystem
            return true;
        }
        
        // check extension
        String ext = getExtension(file);
        if ( ext != null ){
    
            boolean result = false;
            
            for (int i=0; i<rdfExtensions.length; i++){
                if ( ext.equals( rdfExtensions[i] ) ){
                    // found a match!
                    result = true;
                    break;
                }
            }
            return result;
        }
        // if there is no extension...
        return false;    
    }

    
    /**
        Returns description of this filter.
	This short description is intended to be displayed in a
	filechoosing widget's list of filetype filters.

	@return short descriptive text
    */
    public String getDescription(){
        return "RDF files (*.rdf, *.xrdf, *.xml)";
    }


}
